SystemOrganization addCategory: #'Xpf-Server-Models'!
SystemOrganization addCategory: #'Xpf-Server-Tests'!
SystemOrganization addCategory: #'Xpf-Server-Rest'!
SystemOrganization addCategory: #'Xpf-Server-Rest-Tests'!

RestTest subclass: #XpfRestTest
	instanceVariableNames: 'testProject testUser testUser2 testTask projects users testStory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

XpfRestTest subclass: #XpfRestAuthTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

!XpfRestAuthTest methodsFor: 'as yet unclassified' stamp: 'SR 3/9/2009 10:54'!
testAuthentication

	"Try to get information about a project via http GET request.
	 Verify this by looking into the database directly."

	| response jsonResult|	
	response := self get: 'projects/', testProject id asString request: self class auth.
	self should: response responseCode = 401.
	
	response := self get: 'projects/', testProject id asString request: self class authUser2Pass.
	self should: response responseCode = 403.

	response := self get: 'projects/', testProject id asString request: self class authUserPass.
	self should: response responseCode = 200.

	jsonResult := Json readFrom: response content readStream.
	
 	self 
		should: testProject title = (jsonResult at: 'title');
		should: testProject id = (jsonResult at: 'id').! !

XpfRestTest subclass: #XpfRestResourceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:44'!
testDeleteOneProject
	"Try to delete a project via http DELETE request. This operation should be idempotent. 
	Verify this by looking into the database directly."

	| projectCount response |
	projectCount := XpfProject all size.
	response := self delete: 'projects/', testProject id asString.
	
	self 
		should: response responseCode = 200;
		should: XpfProject all size = (projectCount - 1);
		shouldnt: (XpfProject all anySatisfy: [:aProject| aProject id = testProject id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:44'!
testDeleteOneTask
	"Try to delete a task from testProject via http DELETE request.
	 This operation should be idempotent. Verify this by looking into the database directly."

	| taskCount response |
	taskCount := testProject tasks size.
	response := self delete: ('projects/{1}/tasks/{2}' format: {testProject id. testTask id}).
	
	self
		should: response responseCode = 200;
		should: testProject tasks size = (taskCount - 1);
		shouldnt: (testProject tasks anySatisfy: [:task| task id = testTask id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 3/5/2009 13:55'!
testDeleteOneUser
	"Try to delete a user via http DELETE request. This operation should be idempotent. 
	Verify this by looking into the database directly."

	| userCount response |
	userCount := XpfUser all size.
	response := self delete: 'users/', testUser id asString.
	
	self 
		should: response responseCode = 200;
		should: XpfUser all size = (userCount - 1);
		shouldnt: (XpfUser all anySatisfy: [:aUser| aUser id = testUser id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:46'!
testGetAllProjects
	"Try to get a list with rest urls to all projects via http GET request. 
	Verify this by looking into the database directly."

	| response jsonResult restUrl |
	response := self get: 'projects'.	
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self should: XpfProject all size = jsonResult size.
	
	XpfProject all do: [:project |
		restUrl := '/projects/{1}' format:  {project id}.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:47'!
testGetAllTasks
	"Try to get a list with rest urls to all tasks of testProject via http GET request.
	 Verify this by looking into the database directly."

	| response restUrl jsonResult|
	response := self get: ('projects/{1}/tasks' format: {testProject id}).
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self should: testProject tasks size = jsonResult size.
	
	testProject tasks do: [:task |
		restUrl := '/projects/{1}/tasks/{2}' format: {testProject id. task id}.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 3/4/2009 11:15'!
testGetAllUsers
	"Try to get a list with rest urls to all users via http GET request. 
	Verify this by looking into the database directly."

	| response jsonResult restUrl |
	response := self get: 'users'.	
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self should: XpfUser all size = jsonResult size.
	
	XpfUser all do: [:user |
		restUrl := '/users/{1}' format:  {user id}.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mig 3/13/2009 12:06'!
testGetOneProject
	"Try to get information about a project via http GET request.
	 Verify this by looking into the database directly."

	| response restUrl jsonResult|	
	self flag: #NeedsRefactoring.
	response := self get: 'projects/', testProject id asString.
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self 
		should: testProject title = (jsonResult at: 'title');
		should: testProject id = (jsonResult at: 'id');
		should: testProject userStories size = (jsonResult at: 'col userStories') size;
		should: testProject tasks size = (jsonResult at: 'col tasks') size;
		should: testProject developers size = (jsonResult at: 'col developers') size.	
	
	testProject userStories do: [:story |
		restUrl := '/projects/{1}/userStories/{2}' format:  {testProject id. story id}.
		self should: ((jsonResult at: 'col userStories') includes: restUrl)].

	testProject tasks do: [:task |
		restUrl := '/projects/{1}/tasks/{2}' format:  {testProject id. task id}.
		self should: ((jsonResult at: 'col tasks') includes: restUrl)].
	
	testProject developers do: [:developer |
		restUrl := '/users/{1}' format:  {developer id}.
		self should: ((jsonResult at: 'col developers') includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mfp 3/20/2009 10:59'!
testGetOneTask
	"Try to get information about a task from testProject via http GET request.
	 Verify this by looking into the database directly."

	| response restUrl jsonResult |	
	self flag: #NeedsRefactoring.
	response := self get: ('projects/{1}/tasks/{2}' format: {testProject id. testTask id}).
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self
		should: testTask title = (jsonResult at: 'title');
		should: testTask id = (jsonResult at: 'id');
		should: testTask explanation = (jsonResult at: 'explanation');
		should: testTask timeEstimated = (jsonResult at: 'timeEstimated');
		should: testTask timeActual = (jsonResult at: 'timeActual');
		should: testTask isDone = (jsonResult at: 'isDone');
		should: ('/projects/', testTask project id asString) = (jsonResult at: 'ref project');
	
		should: testTask userStories size = (jsonResult at: 'col userStories') size;
		should: testTask developers size = (jsonResult at: 'col developers') size.	
	
	testTask userStories do: [:story |
		restUrl := '/projects/{1}/userStories/{2}' format:  {testProject id. story id}.
		self should: ((jsonResult at: 'col userStories') includes: restUrl)].
	
	testTask developers do: [:developer |
		restUrl := '/users/{1}' format:  {developer id}.
		self should: ((jsonResult at: 'col developer') includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/5/2009 15:46'!
testPostAddProject
	"Try to create a new project via http POST request. Verify this by looking into the database directly."

	| response projectCount newProject |
	projectCount := XpfProject all size.
	response := self post: 'projects' content: (self jsonDataFrom: {'title' -> 'new test project'}).

	self should: XpfProject all size = (projectCount + 1).

	newProject := XpfProject all detect: [:aProject | 
		aProject id = response content asURI pathComponents last asNumber].
	self 
		should: newProject title = 'new test project';
		should: response content = ('/projects/{1}' format: {newProject id})! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:54'!
testPostAddTask
	"Try to create a new task to testProject via http POST request.
	 Verify this by looking into the database directly."
	
	| response tasksCount newTask |
	self flag: #CreateFixtures.
	tasksCount := testProject tasks size.
	response := self
		post: ('projects/{1}/tasks' format: {testProject id})
		content: (self jsonDataFrom: {'title' -> 'new test task'. 'explanation' -> 'This is a test task.'. 'timeEstimated' -> 42}).
	
	self should: response responseCode = 200.
	
	response := response content.
	
	self should: testProject tasks size = (tasksCount + 1).
	
	newTask := testProject tasks detect:
		[:task | task id = response asURI pathComponents last asNumber].
	
	self
		should: response = ('/projects/{1}/tasks/{2}' format: {testProject id. newTask id});
		should: newTask title = 'new test task';
		should: newTask explanation = 'This is a test task.';
		should: newTask timeEstimated = 42;
		should: newTask project = testProject.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 3/6/2009 14:03'!
testPostAddUser
	"Try to create a new user via http POST request. Verify this by looking into the database directly."

	| response userCount newUser |
	userCount := XpfUser all size.
	response := self post: 'users' content: (self jsonDataFrom: {'userName' -> 'New User'. 'login' -> 'new.user@email.de'. 'password' -> nil}).

	self should: XpfUser all size = (userCount + 1).

	newUser := XpfUser all detect: [:aProject | 
		aProject id = response content asURI pathComponents last asNumber].
	self 
		should: newUser userName = 'New User';
		should: newUser login = 'new.user@email.de';
		should: response content = ('/users/{1}' format: {newUser id})! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'cs 3/9/2009 13:28'!
testPostAttachDeveloperToProject
	"Try to attach testUser to testProject via http POST request"
	
	| response developersCount testUser2 |
	testUser2 := XpfUser addNewUser:
		(Dictionary newFrom: {'userName' -> 'Nummer 2'. 'login' -> 'No2'. 'password' -> 'secret'}).
	developersCount := testProject developers size.
	response := self
		post: ('projects/{1}/developers' format: {testProject id})
		content: ('"/users/{1}"' format: {testUser2 id}).
	
	self 
		should: response responseCode = 200;
		should: testProject developers size = (developersCount + 1);
		should: (testProject developers includes: testUser2);
		should: response content = ('/users/{1}' format: {testUser2 id}).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mfp 3/6/2009 17:11'!
testPostAttachDeveloperToTask
	"Try to attach testUser to testTask via http POST request.
	 Verify this by looking into the database directly."
	
	| response developersCount |
	developersCount := testTask developers size.
	response := self
		post: ('projects/{1}/tasks/{2}/developers' format: {testProject id. testTask id})
		content: ('"/users/{1}"' format: {testUser id}).
	
	self 
		should: response responseCode = 200;
		should: testTask developers size = (developersCount + 1);
		should: (testTask developers includes: testUser);
		should: response content = ('/users/{1}' format: {testUser id}).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'topa 3/3/2009 22:55'!
testPutOneProject
	"Try to update information for a project via http PUT request.
	 This operation should be idempotent. Verify this by looking into the database directly."
	
	| projectCount response developersCount tasksCount userStoriesCount |
	projectCount := XpfProject all size.
	userStoriesCount := testProject userStories size.
	developersCount := testProject developers size.
	tasksCount := testProject tasks size.
	
	2 timesRepeat: [	
		response := self
			put: 'projects/', testProject id asString
			content: (self jsonDataFrom: {'title' -> 'New Title'}).
			
		self 
			should: response responseCode = 200;
			should: XpfProject all size = projectCount;
			should: testProject title = 'New Title';
			should: testProject userStories size = userStoriesCount;
			should: testProject developers size = developersCount;
			should: testProject tasks size = tasksCount].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mfp 3/20/2009 10:59'!
testPutOneTask
	"Try to update information for a task of testProject via http PUT request.
	 This operation should be idempotent. Verify this by looking into the database directly."
	
	| taskCount response developersCount userStoriesCount |
	self flag: #CreateFixtures.
	taskCount := testProject tasks size.
	userStoriesCount := testTask userStories size.
	developersCount := testTask developers size.
	
	2 timesRepeat: [
		response := self
			put: ('projects/{1}/tasks/{2}' format: {testProject id. testTask id})
			content: (self jsonDataFrom: {
				'title' -> 'New Title'.
				'explanation' -> 'New Explanation'.
				'timeEstimated' -> 25.
				'timeActual' -> 50.
				'isDone' -> true}).
			
		self 
			should: response responseCode = 200;
			should: testProject tasks size = taskCount;
			should: testTask title = 'New Title';
			should: testTask explanation = 'New Explanation';
			should: testTask timeEstimated = 25;
			should: testTask timeActual = 50;
			should: testTask userStories size = userStoriesCount;
			should: testTask developers size = developersCount;
			should: testTask project = testProject;
			should: testTask isDone].! !

!XpfRestTest class methodsFor: 'testing' stamp: 'topa 3/5/2009 13:08'!
isAbstract
	
	^ self == XpfRestTest
! !

!XpfRestTest methodsFor: 'actions' stamp: 'mt 3/12/2009 13:10'!
backup

	XpfProject useTestData: true.
	XpfUser useTestData: true.
	
	XpfProject projects: Set new.
	XpfUser users: Set new.! !

!XpfRestTest methodsFor: 'actions' stamp: 'mt 3/12/2009 13:10'!
restore

	XpfProject useTestData: false.
	XpfUser useTestData: false.! !

!XpfRestTest methodsFor: 'running' stamp: 'cs 3/18/2009 11:08'!
setUp
	"Install a handler for http service. Add a test project to the database."

	self flag: #SplitUp.
	super setUp.
	self backup.
	RestHttpService resource installHandler: XpfRestResourceHttpHandler on: '/'.
	testProject := XpfProject new title: 'Test Project'.
	testUser := XpfUser new login: self class login; password: self class password. 
	testUser2 := XpfUser new login: self class login2; password: self class password.
	testStory := XpfUserStory new title: 'Test UserStory'.
	testTask := XpfTask new
				title: 'Test Task';
				explanation: 'This is a test task.';
				timeEstimated: 12;
				timeActual: 15.
	XpfProject addProject: testProject.
	XpfUser addUser: testUser.
	XpfUser addUser: testUser2.
	testProject addUserStory: testStory.
	testStory addTask: testTask.
	testProject addDeveloper: testUser.! !

!XpfRestTest methodsFor: 'running' stamp: 'topa 3/3/2009 18:46'!
tearDown

	self restore.
	super tearDown.
! !

!Object methodsFor: '*xpf-server-models' stamp: 'mfp 3/6/2009 11:45'!
isXpfEntity
	^ false! !

Object subclass: #XpfEntity
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfEntity class
	instanceVariableNames: 'lastId'!

!XpfEntity commentStamp: 'mt 2/12/2009 12:20' prior: 0!
I am the abstract superclass of all entities in the system. These entities could be user stories, tasks, developers, themes, ...!

XpfEntity class
	instanceVariableNames: 'lastId'!

!XpfEntity class methodsFor: 'description' stamp: 'topa 3/3/2009 22:15'!
collectionPropertyNames

	^ #()! !

!XpfEntity class methodsFor: 'private' stamp: 'mt 2/25/2009 10:22'!
incrementId

	self pvtLastId: self pvtLastId + 1.
	^ self pvtLastId! !

!XpfEntity class methodsFor: 'initialization' stamp: 'mt 2/25/2009 10:22'!
initialize

	super initialize.
	self pvtLastId: 0.! !

!XpfEntity class methodsFor: 'description' stamp: 'topa 3/3/2009 22:15'!
primitivePropertyNames

	^ #(id)! !

!XpfEntity class methodsFor: 'accessing' stamp: 'mt 2/25/2009 10:32'!
pvtLastId

	^ lastId ifNil: [lastId := 0]! !

!XpfEntity class methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:15'!
pvtLastId: anObject

	lastId := anObject.! !

!XpfEntity class methodsFor: 'description' stamp: 'topa 3/3/2009 22:15'!
referencePropertyNames

	^ #()! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:13'!
id

	^ id! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:13'!
id: anObject

	id := anObject.! !

!XpfEntity methodsFor: 'authorization' stamp: 'mfp 3/6/2009 11:18'!
isMethod: aMethodString allowedFor: aUser
	"returns whether the method is allowed for the specified user;	may be overwritten in subclasses"

	^ true! !

!XpfEntity methodsFor: 'testing' stamp: 'mfp 3/6/2009 11:45'!
isXpfEntity
	^ true! !

!XpfEntity methodsFor: 'printing' stamp: 'SR 3/20/2009 22:00'!
jsonWriteOn: aStream
	"Write its own properties onto the stream:
	'{'title'->'Just an entity', 
	  'ref project' -> '/projects/3', 
	  'col userStories' -> ['/..', '/..']}'
	'ref' indicates a reference to another resource
	'col' indicates a collection of other resources"

	| result |
	result := Dictionary newFrom: (self class primitivePropertyNames collect: [ :aSymbol |
			aSymbol -> (self perform: aSymbol)]).
	result addAll: (Dictionary newFrom: (self class referencePropertyNames collect: [ :aSymbol | 
			'ref ', aSymbol -> ((self perform: aSymbol)
				ifNil: [nil]
				ifNotNil: [(self perform: aSymbol) restUri])])).
	result addAll: (Dictionary newFrom: (self class collectionPropertyNames collect: [ :aSymbol |
			'col ', aSymbol -> ((self perform: aSymbol) collect: [:each | each restUri])])).
	result jsonWriteOn: aStream.
	! !

!XpfEntity methodsFor: 'printing' stamp: 'mt 2/25/2009 10:12'!
restUri

	^ (XpfRestResourceHttpHandler uriMapping at: self class) value: self! !

XpfEntity subclass: #XpfIteration
	instanceVariableNames: 'title userStories project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 3/21/2009 14:49'!
collectionPropertyNames

	^ super collectionPropertyNames, #(userStories)! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'mig 3/21/2009 16:36'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: [])! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 3/20/2009 21:51'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title)! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 3/20/2009 21:51'!
referencePropertyNames

	^ super referencePropertyNames, #(project)! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:55'!
addNewUserStory: aDictionary

	^ self addUserStory:	(XpfUserStory newFrom: aDictionary)! !

!XpfIteration methodsFor: 'as yet unclassified' stamp: 'SR 3/20/2009 21:41'!
addUserStory: aUserStory 

	self userStories add: aUserStory ifPresent: [^ aUserStory].
	self project addUserStory: aUserStory.
	^ aUserStory iteration: self; yourself! !

!XpfIteration methodsFor: 'as yet unclassified' stamp: 'mig 3/21/2009 11:45'!
delete

	self userStories do: [:us | us iteration: nil].
	self project removeIteration: self.! !

!XpfIteration methodsFor: 'as yet unclassified' stamp: 'mig 3/21/2009 11:42'!
initialize

	super initialize.
	self userStories: OrderedCollection new.! !

!XpfIteration methodsFor: 'as yet unclassified' stamp: 'SR 3/20/2009 20:56'!
printString

	^ self title ifNil: ['No Title']! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:56'!
project
	^ project! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:56'!
project: anObject
	project := anObject! !

!XpfIteration methodsFor: 'as yet unclassified' stamp: 'mig 3/21/2009 17:03'!
removeUserStory: aUserStory

	self userStories remove: aUserStory ifAbsent: [^ aUserStory].
	^ aUserStory iteration: nil; yourself! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:54'!
title
	^ title! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:54'!
title: anObject
	title := anObject! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:54'!
userStories
	^ userStories! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 3/20/2009 20:54'!
userStories: anObject
	userStories := anObject! !

XpfEntity subclass: #XpfProject
	instanceVariableNames: 'title userStories tasks developers iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfProject class
	instanceVariableNames: 'projects testProjects useTestData'!

XpfProject class
	instanceVariableNames: 'projects testProjects useTestData'!

!XpfProject class methodsFor: 'add-remove' stamp: 'mt 2/25/2009 10:27'!
addNewProject: aDictionary

	^ self addProject: (self newFrom: aDictionary)! !

!XpfProject class methodsFor: 'add-remove' stamp: 'cs 3/9/2009 14:48'!
addProject: aProject

	(self projects includes: aProject) ifTrue: [^ aProject].
	aProject id: self incrementId.
	^ self projects add: aProject! !

!XpfProject class methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
all

	^ self projects! !

!XpfProject class methodsFor: 'description' stamp: 'SR 3/20/2009 21:47'!
collectionPropertyNames

	^ super collectionPropertyNames, #(userStories developers tasks iterations)! !

!XpfProject class methodsFor: 'initialization' stamp: 'mt 3/12/2009 13:02'!
initialize

	super initialize.
	self
		useTestData: false;
		projects: Set new;
		testProjects: Set new.! !

!XpfProject class methodsFor: 'instance creation' stamp: 'mt 3/16/2009 11:00'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title')! !

!XpfProject class methodsFor: 'description' stamp: 'topa 3/3/2009 22:26'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title)! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:54'!
projects

	^ self useTestData
		ifTrue: [self testProjects]
		ifFalse: [projects]! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:57'!
projects: aCollection

	self useTestData
		ifTrue: [self testProjects: aCollection]
		ifFalse: [projects := aCollection].! !

!XpfProject class methodsFor: 'add-remove' stamp: 'mfp 3/6/2009 14:21'!
removeProject: aProject
	"if absent, be silent."

	^ self projects remove: aProject ifAbsent: [^ aProject]! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:50'!
testProjects
	^ testProjects! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:50'!
testProjects: anObject
	testProjects := anObject! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:50'!
useTestData
	^ useTestData! !

!XpfProject class methodsFor: 'testing' stamp: 'mt 3/12/2009 12:45'!
useTestData: aBoolean

	useTestData := aBoolean! !

!XpfProject methodsFor: 'add-remove' stamp: 'cs 3/9/2009 13:12'!
addDeveloper: aUser

	self developers add: aUser ifPresent: [^ aUser].
	XpfUser addUser: aUser.
	^ aUser addProject: self; yourself! !

!XpfProject methodsFor: 'add-remove' stamp: 'SR 3/20/2009 21:15'!
addIteration: aIteration 

	self iterations add: aIteration ifPresent: [^ aIteration].
	aIteration
		project: self;
		id: aIteration class incrementId.
	^ aIteration! !

!XpfProject methodsFor: 'add-remove' stamp: 'SR 3/20/2009 21:15'!
addNewIteration: aDictionary

	^ self addIteration:	(XpfIteration newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 2/25/2009 09:56'!
addNewTask: aDictionary

	^ self addTask: (XpfTask newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 2/25/2009 09:57'!
addNewUserStory: aDictionary

	^ self addUserStory:	(XpfUserStory newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 3/13/2009 15:53'!
addTask: aTask

	self tasks add: aTask ifPresent: [^ aTask].
	aTask
		project: self;
		id: aTask class incrementId.
	^ aTask! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 3/13/2009 15:55'!
addUserStory: aUserStory 

	self userStories add: aUserStory ifPresent: [^ aUserStory].
	aUserStory
		project: self;
		id: aUserStory class incrementId.
	^ aUserStory! !

!XpfProject methodsFor: 'deleting' stamp: 'mt 2/25/2009 10:26'!
delete

	self class removeProject: self.! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:22'!
developers

	^ developers! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:22'!
developers: anObject

	developers := anObject.! !

!XpfProject methodsFor: 'initialization' stamp: 'SR 3/20/2009 21:39'!
initialize

	super initialize.
	self
		userStories: OrderedCollection new;
		tasks: OrderedCollection new;
		developers: OrderedCollection new;
		iterations: OrderedCollection new.
	! !

!XpfProject methodsFor: 'as yet unclassified' stamp: 'cs 3/18/2009 15:28'!
isMethod: aMethodString allowedFor: aUser

	^ self developers includes: aUser! !

!XpfProject methodsFor: 'accessing' stamp: 'SR 3/20/2009 21:14'!
iterations
	^ iterations! !

!XpfProject methodsFor: 'accessing' stamp: 'SR 3/20/2009 21:14'!
iterations: anObject
	iterations := anObject! !

!XpfProject methodsFor: 'printing' stamp: 'mt 2/25/2009 10:16'!
printString

	^ self title ifNil: ['No Title']! !

!XpfProject methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:27'!
removeDeveloper: aUser 

	self developers remove: aUser ifAbsent: [^ aUser].
	^ aUser removeProject: self; yourself! !

!XpfProject methodsFor: 'add-remove' stamp: 'SR 3/20/2009 21:16'!
removeIteration: aIteration

	self iterations remove: aIteration ifAbsent: [^ aIteration].
	aIteration project: nil.
	^ aIteration! !

!XpfProject methodsFor: 'add-remove' stamp: 'SR 3/19/2009 21:45'!
removeTask: aTask

	self tasks remove: aTask ifAbsent: [^ aTask].
	self userStories do: [:us | us tasks remove: aTask ifAbsent: [^ aTask]].
	aTask project: nil.
	^ aTask! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 2/25/2009 10:09'!
removeUserStory: aUserStory

	self userStories remove: aUserStory ifAbsent: [^ aUserStory].
	aUserStory project: nil.
	^ aUserStory! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
tasks

	^ tasks! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
tasks: anObject

	tasks := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
title

	^ title! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
title: anObject

	title := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
userStories

	^ userStories! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
userStories: anObject

	userStories := anObject.! !

XpfEntity subclass: #XpfTask
	instanceVariableNames: 'title explanation userStories timeEstimated timeActual project developers isDone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfTask commentStamp: 'mt 2/12/2009 12:22' prior: 0!
I am a task in tems of Extreme Programming.!

!XpfTask class methodsFor: 'description' stamp: 'topa 3/3/2009 22:32'!
collectionPropertyNames

	^ super collectionPropertyNames, #(userStories developers)! !

!XpfTask class methodsFor: 'instance creation' stamp: 'mfp 3/20/2009 10:55'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		explanation: (aDictionary at: 'explanation' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		isDone: (aDictionary at: 'isDone' ifAbsent: [false])! !

!XpfTask class methodsFor: 'description' stamp: 'mfp 3/20/2009 10:57'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title explanation timeEstimated timeActual isDone)! !

!XpfTask class methodsFor: 'description' stamp: 'mig 3/21/2009 12:00'!
referencePropertyNames

	^ super referencePropertyNames, #(project)! !

!XpfTask methodsFor: 'add-remove' stamp: 'cs 3/18/2009 15:28'!
addDeveloper: aUser

	self developers add: aUser ifPresent: [^ aUser].
	self project addDeveloper: aUser.
	^ aUser addTask: self; yourself! !

!XpfTask methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:29'!
addUserStory: aUserStory

	self userStories add: aUserStory ifPresent: [^ aUserStory].
	self project addUserStory: aUserStory.
	^ aUserStory addTask: self; yourself! !

!XpfTask methodsFor: 'deleting' stamp: 'topa 3/3/2009 22:30'!
delete
	"Deletes this task. Removes all connections first."

	(self userStories union: self developers) do: [:dependant | dependant removeTask: self].
	self project removeTask: self.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
developers

	^ developers! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
developers: anObject

	developers := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
explanation

	^ explanation! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
explanation: anObject

	explanation := anObject.! !

!XpfTask methodsFor: 'initialization' stamp: 'mfp 3/20/2009 11:02'!
initialize

	super initialize.
	self
		isDone: false; 
		userStories: OrderedCollection new;
		developers: OrderedCollection new.! !

!XpfTask methodsFor: 'accessing' stamp: 'mfp 3/20/2009 10:56'!
isDone

	^ isDone! !

!XpfTask methodsFor: 'accessing' stamp: 'mfp 3/20/2009 10:57'!
isDone: anObject

	isDone := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
project

	^ project! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:30'!
project: anObject

	project := anObject.! !

!XpfTask methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:31'!
removeDeveloper: aUser

	self developers remove: aUser ifAbsent: [^ aUser].
	^ aUser removeTask: self; yourself! !

!XpfTask methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:31'!
removeUserStory: aUserStory

	self userStories remove: aUserStory ifAbsent: [^ aUserStory].
	^ aUserStory removeTask: self; yourself! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeActual

	^ timeActual! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeActual: anObject

	timeActual := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeEstimated

	^ timeEstimated! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeEstimated: anObject

	timeEstimated := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
title

	^ title! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:32'!
title: anObject

	title := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:32'!
userStories

	^ userStories! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:32'!
userStories: anObject

	userStories := anObject.! !

XpfEntity subclass: #XpfUser
	instanceVariableNames: 'userName email projects password tasks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfUser class
	instanceVariableNames: 'users adminUser testUsers useTestData'!

!XpfUser commentStamp: 'mt 2/12/2009 12:24' prior: 0!
I am a system or application user. This could be an administrator, webmaster, developer or something like that.!

XpfUser class
	instanceVariableNames: 'users adminUser testUsers useTestData'!

!XpfUser class methodsFor: 'add-remove' stamp: 'mt 3/5/2009 11:17'!
addNewUser: aDictionary

	^ self addUser: (self newFrom: aDictionary)! !

!XpfUser class methodsFor: 'add-remove' stamp: 'cs 3/9/2009 14:38'!
addUser: aUser

	(self users includes: aUser) ifTrue: [^ aUser].
	aUser id: self incrementId.
	^ self users add: aUser! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:08'!
adminUser
	^ adminUser! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:08'!
adminUser: anObject
	adminUser := anObject! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:37'!
all

	^ self users! !

!XpfUser class methodsFor: 'authentication' stamp: 'topa 2/16/2009 17:07'!
authenticate: aLogin by: aPassword
	
	^ self all 
		detect: [ :aUser| (aUser login = aLogin) and: [aUser authenticateBy: aPassword]]
		ifNone: [nil]! !

!XpfUser class methodsFor: 'description' stamp: 'topa 3/3/2009 22:37'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks projects)! !

!XpfUser class methodsFor: 'authentication' stamp: 'SR 3/9/2009 15:46'!
findByLogin: aLoginString

	^ self all detect: [:user | user login = aLoginString]! !

!XpfUser class methodsFor: 'authentication' stamp: 'topa 3/3/2009 22:37'!
hash: aString

	^ SecureHashAlgorithm new hashMessage: aString! !

!XpfUser class methodsFor: 'initialization' stamp: 'mt 3/12/2009 13:02'!
initialize

	super initialize.
	self 
		useTestData: false;
		users: Set new;
		adminUser: (self addNewUser: (Dictionary newFrom: {
			'userName' -> 'Administrator'. 
			'login' -> 'admin@example.com'. 
			'password' -> 'nimda'}));
		testUsers: Set new.! !

!XpfUser class methodsFor: 'instance creation' stamp: 'mt 3/16/2009 11:01'!
newFrom: aDictionary

	^ self new
		userName: (aDictionary at: 'userName' ifAbsent: []);
		login: (aDictionary at: 'login');
		password: (aDictionary at: 'password' ifAbsent: [])! !

!XpfUser class methodsFor: 'description' stamp: 'mt 3/6/2009 13:56'!
primitivePropertyNames

	^ super primitivePropertyNames, #(login userName email password)! !

!XpfUser class methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:38'!
removeUser: aUser
	"if absent, be silent."
	
	^ self users remove: aUser ifAbsent: [^ aUser]
! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:52'!
testUsers
	^ testUsers! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:52'!
testUsers: anObject
	testUsers := anObject! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:52'!
useTestData
	^ useTestData! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:52'!
useTestData: anObject
	useTestData := anObject! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:54'!
users 

	^ self useTestData
		ifTrue: [self testUsers]
		ifFalse: [users]! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 3/12/2009 12:57'!
users: aCollection

	self useTestData
		ifTrue: [self testUsers: aCollection]
		ifFalse: [users := aCollection]! !

!XpfUser methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:33'!
addProject: aProject

	self projects add: aProject ifPresent: [^ aProject].
	XpfProject addProject: aProject.
	^ aProject addDeveloper: self; yourself! !

!XpfUser methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:33'!
addTask: aTask 

	self tasks add: aTask ifPresent: [^ aTask].
	^ aTask addDeveloper: self; yourself
! !

!XpfUser methodsFor: 'authentication' stamp: 'topa 3/3/2009 22:33'!
authenticateBy: aString

	^ self pvtPassword = (self class hash: aString)! !

!XpfUser methodsFor: 'deleting' stamp: 'mt 3/6/2009 13:18'!
delete
	"Deletes this user. Remove all connections first."

	self isAdmin ifTrue: [^self].

	(self tasks union: self projects) do: [:dependant | dependant removeDeveloper: self].
	self class removeUser: self.! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
email

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
email: anObject

	email := anObject.! !

!XpfUser methodsFor: 'initialization' stamp: 'topa 3/3/2009 22:33'!
initialize

	super initialize.
	self 
		projects: OrderedCollection new;
		tasks: OrderedCollection new.! !

!XpfUser methodsFor: 'authentication' stamp: 'mt 3/6/2009 13:08'!
isAdmin

	^ self = self class adminUser! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
lastName

	^ lastName! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
lastName: anObject

	lastName := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 14:05'!
login

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 14:06'!
login: anObject
	"You have to choose a valid email address as a login name."

	email := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:44'!
password

	^ nil! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/6/2009 14:01'!
password: aString

	password := aString ifNotNil: [self class hash: aString].! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:35'!
projects

	^ projects! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:35'!
projects: anObject

	projects := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:35'!
pvtPassword

	^ password! !

!XpfUser methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:36'!
removeProject: aProject 

	self projects remove: aProject ifAbsent: [^ aProject].
	^ aProject removeDeveloper: self; yourself! !

!XpfUser methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:36'!
removeTask: aTask

	self tasks remove: aTask ifAbsent: [^ aTask].
	^ aTask removeDeveloper: self; yourself
! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:36'!
tasks

	^ tasks! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:36'!
tasks: anObject

	tasks := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 13:54'!
userName

	^ userName! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 13:54'!
userName: anObject

	userName := anObject.! !

XpfEntity subclass: #XpfUserStory
	instanceVariableNames: 'title priority timeEstimated tasks timeActual explanation project iteration positionX positionY color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfUserStory commentStamp: 'topa 2/12/2009 14:06' prior: 0!
I am a user story in terms of Extreme Programming!

!XpfUserStory class methodsFor: 'description' stamp: 'topa 3/3/2009 22:40'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks)! !

!XpfUserStory class methodsFor: 'instance creation' stamp: 'mt 3/16/2009 10:59'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		priority: (aDictionary at: 'priority' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: []);
		explanation: (aDictionary at: 'explanation' ifAbsent: [])! !

!XpfUserStory class methodsFor: 'description' stamp: 'SR 3/19/2009 12:39'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title priority timeActual timeEstimated explanation positionX positionY color)! !

!XpfUserStory class methodsFor: 'description' stamp: 'SR 3/20/2009 21:47'!
referencePropertyNames

	^ super referencePropertyNames, #(project iteration)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'mt 3/13/2009 15:08'!
addNewTask: aPropertyDictionary

	^ self addTask: (XpfTask newFrom: aPropertyDictionary)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'SR 3/19/2009 22:39'!
addTask: aTask

	self tasks add: aTask ifPresent: [^ aTask].
	self project addTask: aTask.
	^ aTask addUserStory: self; yourself! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 15:25'!
color

	^ color! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 15:25'!
color: anObject

	color := anObject! !

!XpfUserStory methodsFor: 'deleting' stamp: 'topa 3/3/2009 22:22'!
delete
	"Deletes this user story from the system. Detaches all connections first."

	self tasks do: [:task | task removeUserStory: self].
	self project removeUserStory: self.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
explanation

	^ explanation! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
explanation: anObject

	explanation := anObject.! !

!XpfUserStory methodsFor: 'initialization' stamp: 'SR 3/18/2009 16:08'!
initialize

	super initialize.
	self tasks: OrderedCollection new.
	self positionX: 0.0.
	self positionY: 0.0.
	self color: 'white'.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/20/2009 21:00'!
iteration
	^ iteration! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/20/2009 21:00'!
iteration: anObject
	iteration := anObject! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 11:17'!
positionX
	^ positionX! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 11:17'!
positionX: anObject
	positionX := anObject! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 11:17'!
positionY
	^ positionY! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 11:17'!
positionY: anObject
	positionY := anObject! !

!XpfUserStory methodsFor: 'printing' stamp: 'mt 2/25/2009 10:21'!
printString

	^ self title ifNil: ['No Title']! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
priority

	^ priority! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
priority: anObject

	priority := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
project

	^ project! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
project: anObject

	project := anObject! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'topa 3/3/2009 22:39'!
removeTask: aTask

	self tasks remove: aTask ifAbsent: [^ aTask].
	^ aTask removeUserStory: self; yourself! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
tasks

	^ tasks! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
tasks: anObject

	tasks := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeActual

	^ timeActual! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeActual: anObject

	timeActual := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeEstimated

	^ timeEstimated! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeEstimated: anObject

	timeEstimated := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
title

	^ title! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
title: anObject
	title := anObject.! !

Object subclass: #XpfService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfService class methodsFor: 'as yet unclassified' stamp: 'SR 3/17/2009 16:46'!
start
	RestHttpService start.
	RestHttpService resource
		installHandler: XpfRestBackupHttpHandler on: '/backup';
		installHandler: XpfRestInfoHttpHandler on: '/info';
 	   installHandler: XpfRestResourceHttpHandler on: '/'.! !

!XpfService class methodsFor: 'as yet unclassified' stamp: 'SR 3/9/2009 16:20'!
stop
	RestHttpService stop.! !

RestResourceHttpHandler subclass: #XpfRestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestResourceHttpHandler class methodsFor: 'constants' stamp: 'topa 3/3/2009 22:42'!
rootResources

	^ Dictionary newFrom: {'projects' -> XpfProject. 'users' -> XpfUser}! !

!XpfRestResourceHttpHandler class methodsFor: 'constants' stamp: 'SR 3/20/2009 21:50'!
uriMapping

	^ Dictionary newFrom:{
		XpfProject -> [:project| '/projects/{1}' format: {project id}].
		XpfUser -> [:user| '/users/{1}' format: {user id}].
		XpfTask -> [:task| '/projects/{1}/tasks/{2}' format: {task project id. task id}].
		XpfUserStory -> [:us | '/projects/{1}/userStories/{2}' format: {us project id. us id}].
		XpfIteration -> [:iter | '/projects/{1}/iterations/{2}' format: {iter project id. iter id}]}! !

!XpfRestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'topa 3/5/2009 14:30'!
isAuthorized: anHeaderField

	^ (XpfUser authenticate: anHeaderField userid by: anHeaderField password) notNil
	! !

!XpfRestResourceHttpHandler methodsFor: 'authentication' stamp: 'topa 3/5/2009 13:45'!
needsAuthentication

	^ true! !

!XpfRestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'SR 3/9/2009 15:37'!
requestAllowed: aRequest for: anObject
	| user headerField |
	
	headerField := aRequest headers 
		fieldNamed: 'AUTHORIZATION' 
		ifNone: [RestNotAllowedError signal: 'No authorization information supplied'].
		
	user := XpfUser authenticate: headerField userid by: headerField password.
	
	user isAdmin ifTrue: [^ self].

	(anObject isXpfEntity and: [(anObject isMethod: aRequest methodName allowedFor: user) not])
		ifTrue: [RestNotAllowedError signal: 'Access forbidden for this resource'].! !

RestHttpHandler subclass: #XpfRestBackupHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestBackupHttpHandler methodsFor: 'testing' stamp: 'SR 3/17/2009 16:42'!
isAuthorized: anHeaderField
	| user |
	
	user := XpfUser authenticate: anHeaderField userid by: anHeaderField password.
	^ user notNil and: [user isAdmin]
	! !

!XpfRestBackupHttpHandler methodsFor: 'authentication' stamp: 'SR 3/17/2009 16:41'!
needsAuthentication
	^ true! !

!XpfRestBackupHttpHandler methodsFor: 'request processing' stamp: 'SR 3/17/2009 16:43'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ (super process: anHTTPRequest) ifNil: [
		HTTPResponse ok entity: {XpfProject all. XpfUser all} sixxString.
	]
! !

RestHttpHandler subclass: #XpfRestInfoHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestInfoHttpHandler methodsFor: 'testing' stamp: 'SR 3/9/2009 15:42'!
isAuthorized: anHeaderField

	^ (XpfUser authenticate: anHeaderField userid by: anHeaderField password) notNil
	! !

!XpfRestInfoHttpHandler methodsFor: 'authentication' stamp: 'SR 3/9/2009 15:42'!
needsAuthentication
	^ true! !

!XpfRestInfoHttpHandler methodsFor: 'request processing' stamp: 'SR 3/9/2009 16:04'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ (super process: anHTTPRequest) ifNil: [ | user |
		user := XpfUser findByLogin: (anHTTPRequest headers fieldNamed: 'AUTHORIZATION') userid.
		
		HTTPResponse ok entity: (
			(XpfRestResourceHttpHandler uriMapping at: user class) value: user).
	]
! !

!Collection methodsFor: '*xpf-server' stamp: 'cs 2/19/2009 14:07'!
add: anObject ifPresent: aBlock
	"Add an object if it is absent. Else run the block"

	(self includes: anObject) 
		ifFalse: [^ self add: anObject]
		ifTrue: [^ aBlock value]! !

TestCase subclass: #XpfTestCase
	instanceVariableNames: 'users projects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

XpfTestCase subclass: #XpfAuthTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfAuthTest methodsFor: 'as yet unclassified' stamp: 'mfp 3/6/2009 11:04'!
testUsuallyAllowed
	| entity user |
	entity := XpfEntity new.
	user := XpfUser new.
	
	self should: (entity isMethod: 'METHOD' allowedFor: user).! !

XpfTestCase subclass: #XpfIterationTest
	instanceVariableNames: 'testProject sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfIterationTest methodsFor: 'running' stamp: 'SR 3/20/2009 21:34'!
setUp

	super setUp.
	testProject := XpfProject new title: 'testProject'.
	sut := XpfIteration new 						
		title: 'Iteration 1'.! !

!XpfIterationTest methodsFor: 'tests' stamp: 'mig 3/21/2009 11:40'!
testAttributes

	self
		checkAccessor: #title on: sut with: 'Iteration 2';
		checkAccessor: #userStories on: sut  with: (Set with: XpfUserStory new with: XpfUserStory new);
		checkAccessor: #project on: sut with: testProject.
		
! !

!XpfIterationTest methodsFor: 'tests' stamp: 'mig 3/21/2009 11:44'!
testHasUserStories

	| iterationCount usCount us |
	testProject addIteration: sut.
	iterationCount := testProject iterations size.
	usCount := testProject userStories size.
	
	self should: sut userStories size = 0.
	us := sut addUserStory: XpfUserStory new.
	self
		should: sut userStories size = 1
			description: 'User story count for interation is invalid.';
		should: testProject userStories size = (usCount + 1);
		should: testProject iterations size = iterationCount;
		should: (us iteration = sut).
	
	sut removeUserStory: us.
	self
		should: sut userStories size = 0;
		should: testProject userStories size = (usCount + 1);
		should: testProject iterations size = iterationCount.

	sut delete.
	us delete.! !

!XpfIterationTest methodsFor: 'tests' stamp: 'SR 3/20/2009 21:38'!
testNewAndDelete
	| oldCount |
	oldCount := testProject iterations size.
	testProject addIteration: sut.
	self
		should: testProject iterations size = (oldCount + 1);
		should: (testProject iterations anySatisfy: [:each | each title = sut title]).

	testProject removeIteration: sut.
	self should: testProject iterations size = oldCount! !

!XpfIterationTest methodsFor: 'tests' stamp: 'SR 3/20/2009 21:38'!
testNewFrom

	[XpfIteration newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

XpfTestCase subclass: #XpfProjectTest
	instanceVariableNames: 'testUser sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfProjectTest methodsFor: 'running' stamp: 'cs 3/9/2009 14:33'!
setUp

	super setUp.
	sut := XpfProject new title: 'testProject'.
	testUser := XpfUser new
			login: 'christopher.schuster@student.hpi.uni-potsdam.de';
			userName: 'Christopher Schuster'.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'cs 3/9/2009 14:39'!
testAddExistingUserAsDeveloper
	"Add a user as developer to a project. That user should exist."

	| developerCount userCount developer userId |
	XpfUser addUser: testUser.
	userId := testUser id.
	developerCount := sut developers size.
	userCount := XpfUser all size.
	developer := sut addDeveloper: testUser.

	self
		should: testUser id = userId;
		should: XpfUser all size = userCount;
		should: sut developers size = (developerCount + 1);
		should: (XpfUser all includes: developer);
		should: (sut developers includes: developer).

	sut removeDeveloper: developer.
	self
		should: testUser id = userId;
		should: XpfUser all size = userCount;
		should: sut developers size = developerCount;
		should: (XpfUser all includes: developer);
		shouldnt: (sut developers includes: developer).! !

!XpfProjectTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:40'!
testAttributes

	self checkAccessor: #title on: sut with: 'testProject'.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mfp 3/6/2009 11:14'!
testAuthentication
	self shouldnt: (sut isMethod: 'GET' allowedFor: testUser).

	sut addDeveloper: testUser.
	self should: (sut isMethod: 'GET' allowedFor: testUser).

	sut removeDeveloper: testUser.
	self shouldnt: (sut isMethod: 'GET' allowedFor: testUser).
	! !

!XpfProjectTest methodsFor: 'tests' stamp: 'cs 3/9/2009 14:35'!
testHasDevelopers

	| developerCount userCount developer |
	developerCount := sut developers size.
	userCount := XpfUser all size.
	developer := sut addDeveloper: testUser.

	self
		should: testUser id notNil;
		should: XpfUser all size = (userCount + 1);
		should: sut developers size = (developerCount + 1);
		should: (XpfUser all includes: developer);
		should: (sut developers includes: developer).

	sut removeDeveloper: developer.
	self
		should: XpfUser all size = (userCount + 1);
		should: sut developers size = developerCount;
		should: (XpfUser all includes: developer);
		shouldnt: (sut developers includes: developer).! !

!XpfProjectTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:41'!
testNewAndDelete

	| oldCount |
	oldCount := XpfProject all size.
	XpfProject addProject:sut.
		
	self 
		should: XpfProject all size = (oldCount + 1) description: 'Project count is not valid for database.';
		should: (XpfProject all anySatisfy: [ :each | each title = sut title]).
	
	XpfProject removeProject: sut.
	self should: XpfProject all size = oldCount description: 'Project count is not valid for database after deletion.'.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	| done |
	[XpfProject newFrom: (Dictionary newFrom: {'title'->'testProject'}).]
		on: Error
		do: [self fail].
	
	[XpfProject newFrom: Dictionary new. done := true]
		on: Error
		do: [done := false].

	self shouldnt: done.! !

XpfTestCase subclass: #XpfTaskTest
	instanceVariableNames: 'testProject sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfTaskTest methodsFor: 'running' stamp: 'topa 3/3/2009 23:28'!
setUp

	super setUp.
	testProject := XpfProject new title: 'testProject'.
	sut := XpfTask new title: 'test task'; explanation: 'this is a test task'.
! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mfp 3/20/2009 11:01'!
testAttributes

	self
		shouldnt: sut isDone;
		checkAccessor: #title on: sut with: 'test task';
		checkAccessor: #explanation on: sut with: 'this is a test task';
		checkAccessor: #timeEstimated on: sut with: 10;
		checkAccessor: #timeActual on: sut with: 12;
		checkAccessor: #isDone on: sut with: true;
		checkAccessor: #userStories on: sut with: (Set with: XpfUserStory new with: XpfUserStory new). ! !

!XpfTaskTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:32'!
testHasDevelopers

	| developer developerCount |
	self flag: #NeedsRefactoring.
	developerCount := testProject developers size.
	developer := XpfUser new.
	self should: sut developers size = 0.
	testProject addTask: sut.

	sut addDeveloper: developer.
	self
		should: sut developers size = 1;
		should: testProject developers size = (developerCount + 1);
		should: (developer tasks includes: sut).
	
	sut removeDeveloper: developer.
	self
		should: sut developers size = 0;
		should: testProject developers size = (developerCount + 1).

	sut delete.
	developer delete.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:33'!
testHasUserStories

	| userStory storyCount |
	self flag: #NeedsRefactoring.
	storyCount := testProject userStories size.
	userStory := XpfUserStory new.
	self should: sut userStories size = 0.
	testProject addTask: sut.

	sut addUserStory: userStory.
	self
		should: sut userStories size = 1;
		should: testProject userStories size = (storyCount + 1);
		should: (userStory tasks includes: sut).

	sut removeUserStory: userStory.
	self
		should: sut userStories size = 0;
		should: testProject userStories size = (storyCount + 1).

	sut delete.
	userStory delete! !

!XpfTaskTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:34'!
testNewAndDelete
	| oldCount |
	oldCount := testProject tasks size.

	testProject addTask: sut.
	self
		should: testProject tasks size = (oldCount + 1);
		should: (testProject tasks anySatisfy: [:each | each title = 'test task']).

	sut delete.
	self should: testProject tasks size = oldCount.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	[XpfTask newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

!XpfTestCase class methodsFor: 'testing' stamp: 'topa 3/3/2009 18:42'!
isAbstract

 	^ self name = #XpfTestCase! !

!XpfTestCase methodsFor: 'test-helper' stamp: 'topa 3/3/2009 22:56'!
checkAccessor: aSelector on: aTargetObject with: anObject
	"test getter/setter aSelector of aTargetObject with anObject"
	
	aTargetObject perform: aSelector asMutator with: anObject.
	self should: (aTargetObject perform: aSelector) == anObject
		! !

!XpfTestCase methodsFor: 'running' stamp: 'mt 3/12/2009 13:05'!
setUp

	super setUp.
	
	XpfProject useTestData: true.
	XpfUser useTestData: true.
	
	XpfProject projects: Set new.
	XpfUser users: Set new.! !

!XpfTestCase methodsFor: 'running' stamp: 'mt 3/12/2009 13:07'!
tearDown
	
	XpfProject useTestData: false.
	XpfUser useTestData: false.
	
	super tearDown.! !

XpfTestCase subclass: #XpfUserStoryTest
	instanceVariableNames: 'testProject sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfUserStoryTest methodsFor: 'running' stamp: 'topa 3/3/2009 23:38'!
setUp

	super setUp.
	testProject := XpfProject new title: 'testProject'.
	sut := XpfUserStory new 						
		title: 'User Authentication';
		explanation: 'Users should register and login to the system.';
		priority: 3;
		timeEstimated: 10.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:35'!
testAttributes

	self
		checkAccessor: #title on: sut with: 'US + Tasks verwalten';
		checkAccessor: #explanation on: sut with: 'Dinge tun und Sachen machen';
		checkAccessor: #priority on: sut with: 5;
		checkAccessor: #timeEstimated on: sut  with: 10;
		checkAccessor: #timeActual on: sut  with: 12;
		checkAccessor: #tasks on: sut  with: (Set with: XpfTask new with: XpfTask new). 
		
! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 3/13/2009 15:48'!
testHasTasks

	| taskCount task userStoryCount |
	testProject addUserStory: sut.
	userStoryCount := testProject userStories size.
	taskCount := testProject tasks size.
	
	self should: sut tasks size = 0.
	task := sut addTask: XpfTask new.
	self
		should: sut tasks size = 1
			description: 'Task count for user story is invalid.';
		should: testProject tasks size = (taskCount + 1);
		should: testProject userStories size = userStoryCount;
		should: (task userStories includes: sut).
	
	sut removeTask: task.
	self
		should: sut tasks size = 0;
		should: testProject tasks size = (taskCount + 1);
		should: testProject userStories size = userStoryCount.

	sut delete.
	task delete.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:38'!
testNewAndDelete
	| oldCount |
	oldCount := testProject userStories size.
	testProject addUserStory: sut.
	self
		should: testProject userStories size = (oldCount + 1);
		should: (testProject userStories anySatisfy: [:each | each title = sut title]).

	testProject removeUserStory: sut.
	self should: testProject userStories size = oldCount! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:05'!
testNewFrom

	[XpfUserStory newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

XpfTestCase subclass: #XpfUserTest
	instanceVariableNames: 'sut testProject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfUserTest class methodsFor: 'fixture-data' stamp: 'topa 3/3/2009 22:59'!
login

	^ 'test.user'! !

!XpfUserTest class methodsFor: 'fixture-data' stamp: 'topa 3/3/2009 22:59'!
password

	^ 'TEST'! !

!XpfUserTest methodsFor: 'running' stamp: 'topa 3/3/2009 23:43'!
setUp

	super setUp.
	sut := XpfUser new
				login: self class login;
				password: self class password.
	testProject :=  XpfProject new title: 'xpforums'.! !

!XpfUserTest methodsFor: 'tests' stamp: 'cs 3/9/2009 14:47'!
testAddExistingProjectToUser
	"Add a project to the user. That project should exist."

	| projectCount globalProjectCount projectId |
	XpfProject addProject: testProject.
	projectId := testProject id.
	projectCount := sut projects size.
	globalProjectCount := XpfProject all size.
	
	sut addProject: testProject.
	self
		should: testProject id = projectId;
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = (projectCount + 1);
		should: (XpfProject all includes: testProject);
		should: (sut projects includes: testProject).
	
	sut removeProject: testProject.
	self
		should: testProject id = projectId;
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = projectCount;
		should: (XpfProject all includes: testProject);
		shouldnt: (sut projects includes: testProject).
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/4/2009 14:02'!
testAttributes

	self
		checkAccessor: #login on: sut with: 'marcel.taeumel';
		checkAccessor: #userName on: sut with: 'Marcel Taeumel';
		checkAccessor: #email on: sut with: 'marcel.taeumel@student.hpi.uni-potsdam.de'.! !

!XpfUserTest methodsFor: 'tests' stamp: 'cs 3/9/2009 14:45'!
testHasProjects
	| projectCount globalProjectCount |
	projectCount := sut projects size.
	globalProjectCount := XpfProject all size.
	
	sut addProject: testProject.
	self
		should: testProject id notNil;
		should: XpfProject all size = (globalProjectCount + 1);
		should: sut projects size = (projectCount + 1);
		should: (XpfProject all includes: testProject);
		should: (sut projects includes: testProject).
	
	sut removeProject: testProject.
	self
		should: XpfProject all size = (globalProjectCount + 1);
		should: sut projects size = projectCount;
		should: (XpfProject all includes: testProject);
		shouldnt: (sut projects includes: testProject).
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:47'!
testNewAndDelete

	| oldCount |	
	oldCount := XpfUser all size.	
	self shouldnt: (XpfUser all anySatisfy: [ :each | each login = sut login]).

	XpfUser addUser: sut.
	self 
		should: XpfUser all size = (oldCount + 1);
		should: (XpfUser all anySatisfy: [ :each | each login = sut login]).
	
	XpfUser removeUser: sut.
	self should: XpfUser all size = oldCount.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	| done |
	[XpfUser newFrom: (Dictionary newFrom: {'login'->'test@example.com'}).]
		on: Error
		do: [self fail].
	
	[XpfUser newFrom: Dictionary new. done := true]
		on: Error
		do: [done := false].

	self shouldnt: done.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/6/2009 14:05'!
testPasswordHashing

	self
		"shouldnt: (sut respondsTo: #password);"
		should: sut password = nil;
		should: (sut authenticateBy: self class password);
		shouldnt: (sut authenticateBy: self class password , 'NOT').
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:49'!
testUserAuthenticationFail

	XpfUser addUser: sut.
	self
		should: (XpfUser authenticate: self class login             by: self class password , 'NOT' ) isNil;
		should: (XpfUser authenticate: self class login , 'NOT' by: self class password             ) isNil;
		should: (XpfUser authenticate: self class login , 'NOT' by: self class password , 'NOT') isNil.
	XpfUser removeUser: sut.! !

!XpfUserTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:49'!
testUserAuthenticationSuccess

	XpfUser addUser: sut.
	self should: sut = (XpfUser authenticate: self class login by: self class password).
	XpfUser removeUser: sut! !
XpfEntity initialize!
XpfProject initialize!
XpfUser initialize!
